/*********************************************************************
 *
 *                Microchip USB C18 Firmware Version 1.0
 *
 *********************************************************************
 * FileName:        main.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.30.01+
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the Company) for its PICmicro Microcontroller is intended and
 * supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Rawin Rojvanit       11/19/04    Original.
 ********************************************************************/

/** I N C L U D E S **********************************************************/
#include <p18f4550.h>
#include "system\typedefs.h"                        // Required
#include "system\usb\usb.h"                         // Required
#include "io_cfg.h"                                 // Required
#include "system\usb\usb_compile_time_validation.h" // Optional
#include "user\user.h"                              // Modifiable
#include <timers.h>
#include <math.h>

/** V A R I A B L E S ********************************************************/
#pragma udata 
static unsigned int C_introducir0;
static unsigned int ContadorAjustableTemporizador0;
static unsigned char pasos;

/** P R I V A T E  P R O T O T Y P E S ***************************************/
//extern unsigned char contadorAceleracion;
unsigned int contadorDeceleracion;
unsigned int contadorAceleracion;
unsigned char aceleracion;
unsigned short cont_pulsos;
static void InitializeSystem(void);
void USBTasks(void);
void contenido_interrupcion(void);
void pasoentero1fase(void);
void pasoentero2fase(void);
void mediopaso(void);
void OpenPWM2( char );
void OpenPWM1( char );
void SetDCPWM1( unsigned int );
void SetDCPWM2( unsigned int );

/** V E C T O R  R E M A P P I N G *******************************************/

extern void _startup (void);        // See c018i.c in your C18 compiler dir

#pragma idata 


#pragma code _RESET_INTERRUPT_VECTOR = 0x000800

void _reset (void)
{
    _asm goto _startup _endasm
}
#pragma code

//Acciones cuando se produzcan interrupciones

//En el caso de interrupcion de alta prioridad
#pragma code _HIGH_INTERRUPT_VECTOR = 0x000008
void _high_ISR (void)
{
    _asm goto behitiko_etenaldi _endasm
}

//En el caso de interrupcion de baja prioridad
#pragma code _LOW_INTERRUPT_VECTOR = 0x000018
void _low_ISR (void)
{
   _asm goto behitiko_etenaldi _endasm
}

#pragma code
extern CONTROLMOVIMIENTO controlMovimiento;

//Se vendr aqu cuando se produzca un interrupcion de baja o alta prioridad

#pragma interrupt behitiko_etenaldi

void behitiko_etenaldi(void)
{

if (INTCONbits.TMR0IF==1) //La "banderita" del contador del timer0 que nos avisa, por precaucion, en el caso de que hubiera mas interrupciones
	{  		
	if (aceleracion ==1)
		{	
		if (contadorAceleracion==0)
			{
				OpenPWM2( 0xff );
				OpenPWM1( 0xff );
				SetDCPWM1(controlMovimiento.corrientefaseA);
				SetDCPWM2( controlMovimiento.corrientefaseB );
				ContadorAjustableTemporizador0=controlMovimiento.contadorAcelTemp0;
				contadorAceleracion=contadorAceleracion+1;
			}
			else
			{
				if (contadorAceleracion<=controlMovimiento.pasosAceleracion)
				{
					
					ContadorAjustableTemporizador0=(ContadorAjustableTemporizador0-(2*ContadorAjustableTemporizador0)/(4*contadorAceleracion+1));//Formula de modificacion de C en cada paso
					contadorAceleracion=contadorAceleracion+1;
				}
			}	
			contenido_interrupcion();	
			C_introducir0=(-(ContadorAjustableTemporizador0*3)+65536); //Ajuste del valor de C, para su introduccin en el writetimer0 	//Se multiplica por 0.676 para compensar el error que se produce		
			WriteTimer0(C_introducir0); 	
			INTCONbits.TMR0IF = 0; //Bajamos la banderita 	
	
		}
	if (aceleracion==0)
		{

		if (contadorDeceleracion>0)
			{
					ContadorAjustableTemporizador0=(ContadorAjustableTemporizador0+(2*ContadorAjustableTemporizador0)/(4*contadorDeceleracion+1));//Formula de modificacion de C en cada paso
					contadorDeceleracion=contadorDeceleracion-1;	
					contenido_interrupcion();	
					C_introducir0=(-(ContadorAjustableTemporizador0*3)+65536); //Ajuste del valor de C, para su introduccin en el writetimer0 	//Se multiplica por 0.676 para compensar el error que se produce		
					WriteTimer0(C_introducir0); 	
					INTCONbits.TMR0IF = 0; //Bajamos la banderita 			
			}
		if (contadorDeceleracion==0)
			{
				mLED_3_Off();mLED_4_Off();
				PORTBbits.RB1=0;PORTCbits.RC6=0;PORTAbits.RA5=0;PORTBbits.RB4=0;
				ClosePWM1();ClosePWM2();			
				CloseTimer0;
				INTCONbits.TMR0IF=0;
			}
		

		}
						
	}

//Interrupcin por flanco de bajada en la entrada externa RB0

if (INTCONbits.INT0IF==1)
{

if(controlMovimiento.sentidoGiro==0) //Orden sentido horario por parte de C#
	{
	if(PORTAbits.RA3==1) //El encoder nos dice que es sentido horario
		{
		cont_pulsos=cont_pulsos+1;
		INTCONbits.INT0IF=0;
		}
	else
		{
		cont_pulsos=cont_pulsos-1;
		INTCONbits.INT0IF=0;
		}
	}
else
	{
	if (PORTAbits.RA3==0) //El encoder nos dice que el sentido es antihorario
		{
		cont_pulsos=cont_pulsos-1;
		INTCONbits.INT0IF=0;
		}
	else
		{
		cont_pulsos=cont_pulsos+1;
		INTCONbits.INT0IF=0;
		}
	
	}
}	
}

/** D E C L A R A T I O N S **************************************************/
#pragma code
/******************************************************************************
 * Function:        void main(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Main program entry point.
 *
 * Note:            None
 *****************************************************************************/
void main(void)
{
    InitializeSystem();
    while(1)
    {
        USBTasks();         // USB Tasks
        ProcessIO();        // See user\user.c & .h
    }//end while
}//end main

/******************************************************************************
 * Function:        static void InitializeSystem(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        InitializeSystem is a centralize initialization routine.
 *                  All required USB initialization routines are called from
 *                  here.
 *
 *                  User application initialization routine should also be
 *                  called from here.                  
 *
 * Note:            None
 *****************************************************************************/
static void InitializeSystem(void)
{
    ADCON1 |= 0x00;                 // Default all pins to digital(0x0f) baina nik aldatu dut analogikotatik digitaletara
    
    #if defined(USE_USB_BUS_SENSE_IO)
    tris_usb_bus_sense = INPUT_PIN; // See io_cfg.h
    #endif
    
    #if defined(USE_SELF_POWER_SENSE_IO)
    tris_self_power = INPUT_PIN;
    #endif
    
    mInitializeUSBDriver();         // See usbdrv.h
    
    UserInit();                     // See user.c & .h

}//end InitializeSystem

/******************************************************************************
 * Function:        void USBTasks(void)
 *
 * PreCondition:    InitializeSystem has been called.
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Service loop for USB tasks.
 *
 * Note:            None
 *****************************************************************************/
void USBTasks(void)
{
    /*
     * Servicing Hardware
     */
    USBCheckBusStatus();                    // Must use polling method
    if(UCFGbits.UTEYE!=1)
        USBDriverService();                 // Interrupt or polling method

}// end USBTasks

/******************************************************************************
 * Function:        void contenido_interrupcion(void)
 *
 * PreCondition:    
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Da funcionalidad a las interrupciones del Timer0 que 
 *					controlan el tiempo de excitacion de los devanados
 *
 * Note:            None
 *****************************************************************************/


void contenido_interrupcion()
{
switch(controlMovimiento.tipoMovimiento)	
{
case PASOENTERO1FASE:
pasoentero1fase();
break;
case PASOENTERO2FASE:
pasoentero2fase();
break;
case MEDIOPASO:
mediopaso();
break;
case MICROPASO:
break;
}
}

/** EOF main.c ***************************************************************/