/*********************************************************************
 *
 *                Microchip USB C18 Firmware Version 1.0
 *
 *********************************************************************
 * FileName:        encadain.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.30.01+
 * 
 *
 * Este programa contiene todas las diversas funciones que pueden	
 *	ejecutarse cada vez que entra la interrupcin de la tarjeta o	
 *	que impliquen el giro del motor paso a paso.	.
 ********************************************************************

/** I N C L U D E S **********************************************************/
#include <p18f4550.h>
#include <usart.h>
#include "system\typedefs.h"
#include <ADC.h>
#include "system\usb\usb.h"
#include "io_cfg.h"             // I/O pin mapping
#include "user\user.h"
#include <timers.h>
#include <portb.h>
#include <pwm.h>
/** V A R I A B L E S ********************************************************/
#pragma udata 

/** P R I V A T E  P R O T O T Y P E S ***************************************/
void PWMdutyCorrientesFases(byte *);
void pasoentero1fase(void);
#pragma idata
static unsigned int contar=1;




/** D E C L A R A T I O N S **************************************************/
#pragma code
extern CONTROLMOVIMIENTO controlMovimiento;

//extern struct CONTROLMOVIMIENTO *_pointerCorrientefases;

/****************************************************************************	
	Nombre:		pasoentero1fase				
	Recibe:		nada						
	Devuelve:	nada						
	Esta funcin se encarga de excitar al motor paso a paso segn	
	una secuencia de pasos enteros (7.5 grados) en la cual esta solo
	1 fase excitada simultneamente. Se puede girar	
	en sentido horario y antihorario.				
	Nota: Se utilizan los pins RB1 y RB3 del PIC para desinhibir
	los transistores de las Fase B y A respectivamente.
	Se inhiben con la seal a cero.	
*****************************************************************************/

void pasoentero1fase()
{


if (controlMovimiento.sentidoGiro==0){ //Sentido horario

if (contar==1) 	{
mLED_3_Off();
mLED_4_Off();
PORTBbits.RB1=1;	//Inhibit_A
PORTAbits.RA5=0;	//Inhibit_B
PORTCbits.RC6=0;	//Phase_A
PORTBbits.RB4=0;	//Phase_B
}//PORTB =0b00011110;
if (contar==2) {
mLED_3_On();
mLED_4_On();
PORTBbits.RB1=0; 	//Inhibit_A
PORTAbits.RA5=1;	//Inhibit_B
PORTCbits.RC6=0;	//Phase_A
PORTBbits.RB4=0;	//Phase_B
}//	PORTB =0b00011010;
if (contar==3){
mLED_3_Off();
mLED_4_On();
PORTBbits.RB1=1;	//Inhibit_A
PORTAbits.RA5=0;	//Inhibit_B
PORTCbits.RC6=1;	//Phase_A
PORTBbits.RB4=0;	//Phase_B
}// 	PORTB =0b00001010;
if (contar==4) 	
{
mLED_3_On();
mLED_4_Off();
PORTBbits.RB1=0;	//Inhibit_A
PORTAbits.RA5=1;	//Inhibit_B
PORTCbits.RC6=0;	//Phase_A
PORTBbits.RB4=1;	//Phase_B
}
}
else	//sentido antihorario
{
if (contar==1) 	{
mLED_3_Off();
mLED_4_Off();
PORTBbits.RB1=1; //Inhibit_A
PORTAbits.RA5=0; //Inhibit_B
PORTCbits.RC6=0; //Phase_A
PORTBbits.RB4=1; //Phase_B
}//PORTB =0b00011110;
if (contar==2) {
mLED_3_On();
mLED_4_Off();
PORTBbits.RB1=0;
PORTAbits.RA5=1;
PORTCbits.RC6=1;
PORTBbits.RB4=1;
}//	PORTB =0b00011010;
if (contar==3){
mLED_3_On();
mLED_4_On();
PORTBbits.RB1=1;
PORTAbits.RA5=0;
PORTCbits.RC6=1;
PORTBbits.RB4=0;
}// 	PORTB =0b00001010;
if (contar==4) 	
{
mLED_3_On();
mLED_4_On();
PORTBbits.RB1=0;
PORTAbits.RA5=1;
PORTCbits.RC6=0;
PORTBbits.RB4=0;
}//PORTB =0b00001110;
}
contar++;
if (contar>4) contar=1;
}
//end pasoentero1fase()


/****************************************************************************	
	Nombre:		pasoentero2fase				
	Recibe:		nada						
	Devuelve:	nada						
	Esta funcin se encarga de excitar al motor paso a paso segn	
	una secuencia de pasos enteros (7.5 grados) en la cual siempre	
	estn las dos fases excitadas simultneamente. Se puede girar	
	en sentido horario y antihorario.				
	Nota: Se utilizan los pins RB1 y RB3 del PIC para desinhibir
	los transistores de las Fase B y A respectivamente.
	Se inhiben con la seal a cero.	
*****************************************************************************/
void pasoentero2fase(){

//se mandan las corrientes de referencia para las dos fases del motor
// Timer2 ya esta inicializado, las corrientes enl a fases A y B puestas
//En user.c PORTB para controlar RBO=INT0=interrupciones encoder RB1=INHIBIT_A 
//RC6=PWM_A RB3=INHIBIT_B RB4=PWM_B

/*Cuando haya que tomar datos del encoder para grficos en pantalla PC
Enable interrupts for PORTB0 pin para la permitir las interrupciones del encoder.  
*/
PORTBbits.RB1=1; //Habilitamos los mosfests de la rama superior de la fase A
PORTAbits.RA5=1; //Habilitamos los mosfests de la rama superior de la fase B

if (controlMovimiento.sentidoGiro==0){ //Sentido horario

if (contar==1) {
mLED_3_Off();
mLED_4_Off();
PORTCbits.RC6=1;
PORTBbits.RB4=1;
}//PORTB =0b00011110;
if (contar==2) {
mLED_3_On();
mLED_4_Off()
PORTCbits.RC6=0;
PORTBbits.RB4=1;
}	//PORTB =0b00011010;
if (contar==3){
mLED_3_On();
mLED_4_On();
PORTCbits.RC6=0;
PORTBbits.RB4=0;
}	//PORTB =0b00001010;
if (contar==4){
mLED_3_Off();
mLED_4_On();
PORTCbits.RC6=1;
PORTBbits.RB4=0;
}// 	PORTB =0b00001110;
}
else	//sentido antihorario
{
if (contar==4) 	{
mLED_3_Off();
mLED_4_Off();
PORTCbits.RC6=1;
PORTBbits.RB4=1;
}//PORTB =0b00011110;
if (contar==3) {
mLED_3_On();
mLED_4_Off();
PORTCbits.RC6=0;
PORTBbits.RB4=1;
}//	PORTB =0b00011010;
if (contar==2){
mLED_3_On();
mLED_4_On();
PORTCbits.RC6=0;
PORTBbits.RB4=0;
}// 	PORTB =0b00001010;
if (contar==1) 	
{
mLED_3_On();
mLED_4_On();
PORTCbits.RC6=1;
PORTBbits.RB4=0;
}//PORTB =0b00001110;
}
contar++;
if (contar>4) contar=1;
}
//end pasoentero2fase()




void mediopaso(){

/****************************************************************************	
	Nombre:		mediopaso				
	Recibe:		nada						
	Devuelve:	nada						
	Esta funcin se encarga de excitar al motor paso a paso segn	
	una secuencia de medios pasos (7.5/2 grados). Se puede girar	
	en sentido horario y antihorario.				
	Nota: Se utilizan los pins RB1 y RB3 del PIC para desinhibir
	los transistores de las Fase B y A respectivamente.
	Se inhiben con la seal a cero.	
*****************************************************************************/

if (controlMovimiento.sentidoGiro==0)	//Sentido horario
{ 

if (contar==1) 	
{
mLED_3_On();
mLED_4_On();
PORTBbits.RB1=1;
PORTAbits.RA5=1;
PORTCbits.RC6=1;
PORTBbits.RB4=1;
}

if (contar==2) 	
{
mLED_3_Off();
mLED_4_Off();
PORTBbits.RB1=0;
PORTAbits.RA5=1;
PORTCbits.RC6=0;
PORTBbits.RB4=1;
}
if (contar==3) 	
{
mLED_3_On();
mLED_4_On();
PORTBbits.RB1=1;
PORTAbits.RA5=1;
PORTCbits.RC6=0;
PORTBbits.RB4=1;
}
if (contar==4) 	
{
mLED_3_Off();
mLED_4_Off();
PORTBbits.RB1=1;
PORTAbits.RA5=0;
PORTCbits.RC6=0;
PORTBbits.RB4=0;
}
if (contar==5) 	
{
mLED_3_On();
mLED_4_On();
PORTBbits.RB1=1;
PORTAbits.RA5=1;
PORTCbits.RC6=0;
PORTBbits.RB4=0;
}
if (contar==6) 	
{
mLED_3_Off();
mLED_4_Off();
PORTBbits.RB1=0;
PORTAbits.RA5=1;
PORTCbits.RC6=1;
PORTBbits.RB4=0;
}
if (contar==7) 	
{
mLED_3_On();
mLED_4_On();
PORTBbits.RB1=1;
PORTAbits.RA5=1;
PORTCbits.RC6=1;
PORTBbits.RB4=0;
}
if (contar==8) 	
{
mLED_3_Off();
mLED_4_Off();
PORTBbits.RB1=1;
PORTAbits.RA5=0;
PORTCbits.RC6=1;
PORTBbits.RB4=1;
}

}

else //Sentido antihorario
{

if (contar==1) 	
{
mLED_3_On();
mLED_4_On();
PORTBbits.RB1=1;
PORTAbits.RA5=1;
PORTCbits.RC6=1;
PORTBbits.RB4=0;
}

if (contar==2) 	
{
mLED_3_Off();
mLED_4_Off();
PORTBbits.RB1=0;
PORTAbits.RA5=1;
PORTCbits.RC6=0;
PORTBbits.RB4=0;
}
if (contar==3) 	
{
mLED_3_On();
mLED_4_On();
PORTBbits.RB1=1;
PORTAbits.RA5=1;
PORTCbits.RC6=0;
PORTBbits.RB4=0;
}
if (contar==4) 	
{
mLED_3_Off();
mLED_4_Off();
PORTBbits.RB1=1;
PORTAbits.RA5=0;
PORTCbits.RC6=0;
PORTBbits.RB4=1;
}
if (contar==5) 	
{
mLED_3_On();
mLED_4_On();
PORTBbits.RB1=1;
PORTAbits.RA5=1;
PORTCbits.RC6=0;
PORTBbits.RB4=1;
}
if (contar==6) 	
{
mLED_3_Off();
mLED_4_Off();
PORTBbits.RB1=0;
PORTAbits.RA5=1;
PORTCbits.RC6=1;
PORTBbits.RB4=1;
}
if (contar==7) 	
{
mLED_3_On();
mLED_4_On();
PORTBbits.RB1=1;
PORTAbits.RA5=1;
PORTCbits.RC6=1;
PORTBbits.RB4=1;
}
if (contar==8) 	
{
mLED_3_Off();
mLED_4_Off();
PORTBbits.RB1=1;
PORTAbits.RA5=0;
PORTCbits.RC6=1;
PORTBbits.RB4=0;
}

}

contar++;
if (contar>8) contar=1;

}


void PWMdutyCorrienteFase(byte *fase){
				
				controlMovimiento.corrientefaseA= *(fase+1);
				controlMovimiento.corrientefaseA=controlMovimiento.corrientefaseA<<8;
				controlMovimiento.corrientefaseA=controlMovimiento.corrientefaseA| (unsigned int) *fase;
				controlMovimiento.corrientefaseB= *(fase+3);
				controlMovimiento.corrientefaseB=controlMovimiento.corrientefaseB<<8;
				controlMovimiento.corrientefaseB=controlMovimiento.corrientefaseB| (unsigned int) *(fase+2);				
}
//end PWMdutyCorrienteFase
void ContadorInicialTemp0(byte *ptrcontadorTemp0){
				
		controlMovimiento.contadorAcelTemp0= *(ptrcontadorTemp0+1);
		controlMovimiento.contadorAcelTemp0=controlMovimiento.contadorAcelTemp0<<8;
		controlMovimiento.contadorAcelTemp0=controlMovimiento.contadorAcelTemp0| (unsigned int) *ptrcontadorTemp0;
		controlMovimiento.contadorDecelTemp0= *(ptrcontadorTemp0+3);
		controlMovimiento.contadorDecelTemp0=controlMovimiento.contadorDecelTemp0<<8;
		controlMovimiento.contadorDecelTemp0=controlMovimiento.contadorDecelTemp0| (unsigned int) *(ptrcontadorTemp0+2);					
}


/*typedef struct CONTROLMOVIMIENTO
{
word contadorAcelTemp0;
word contadorDecelTemp0;
byte sentidoGiro;
byte pasosAceleracion;
byte pasosDeceleracion;
//byte numeroVueltas;
word corrientefaseA;
word corrientefaseB;

enum {
           	PASOENTERO1FASE    	= 0x01,
            PASOENTERO2FASE    	= 0x02,
            MEDIOPASO       	= 0x32,
            MICROPASO			= 0xEE
	 } tipoMovimiento;

}CONTROLMOVIMIENTO;
*/