 /*********************************************************************
 *
 *                Microchip USB C18 Firmware Version 1.0
 *
 *********************************************************************
 * FileName:        user.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.30.01+
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the Company) for its PICmicro Microcontroller is intended and
 * supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Rawin Rojvanit       11/19/04    Original.
 ********************************************************************/

/** I N C L U D E S **********************************************************/
#include <p18f4550.h>
#include <usart.h>
#include "system\typedefs.h"
#include <ADC.h>
#include "system\usb\usb.h"
#include "io_cfg.h"             // I/O pin mapping
#include "user\user.h"
#include <timers.h>
#include <portb.h>
#include <pwm.h>
/** V A R I A B L E S ********************************************************/
#pragma udata

CONTROLMOVIMIENTO controlMovimiento;
byte counter;
byte trf_state;
byte velocL=0;
byte velocH=0;
DATA_PACKET dataPacket;
struct CONTROLMOVIMIENTO *pointerControlmovimiento;
//static unsigned int C_inicio;

/** P R I V A T E  P R O T O T Y P E S ***************************************/
extern unsigned int contadorDeceleracion;
extern unsigned int contadorAceleracion;
extern unsigned char aceleracion;
//extern unsigned char contadorAceleracion;
void ClosePWM1( void );
void ClosePWM2( void );
void OpenPWM1( char );
void SetDCPWM1( unsigned int);
void OpenPWM2( char  );
void SetOutputPWM1 (
        unsigned char , 
        unsigned char );
void SetOutputPWM2 (
        unsigned char , 
        unsigned char );
void SetDCPWM2( unsigned int dutycycle );
void BlinkUSBStatus(void);
void ServiceRequests(void);
void Leer_veloc(void);
void Denboragailu_0_on(struct CONTROLMOVIMIENTO *);
void Denboragailu_0_off(void);
/** D E C L A R A T I O N S **************************************************/
#pragma code
void UserInit(void)
{
 mInitAllLEDs();

//configuramos como entrada/salida los puertos utilizados 
TRISB=0b00000001; 
TRISCbits.TRISC6=0;
TRISCbits.TRISC7=0;
TRISAbits.TRISA5=0;
TRISAbits.TRISA3=1;
//Se programa Timer2 para con PWM obtener las referencias de corriente
OpenTimer2( TIMER_INT_OFF &T2_PS_1_16 &T2_POST_1_1 );
ADCON1 =0x0E;     //configures Vref to Vdd which is +5V in this case
INTCON = 0b11110000; 
INTCON2= 0b00000000;
INTCON3= 0b00000000;
INTCON2bits.TMR0IP =1;			//Configuramos la interrupcio del timer0 con alta prioridad
RCONbits.IPEN = 1; 
PORTBbits.RB2=1;
}//end UserInit





/******************************************************************************
 * Function:        void Leer_veloc (void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Funcion para lectura del contador de impulsos          
 *
 * Note:            None
 *****************************************************************************/

void Leer_veloc(void)
{
 velocL=cont_pulsos;
 velocH=cont_pulsos>>8;
 cont_pulsos=0;
}

/******************************************************************************
 * Function:        void Denboragailu_0_on (void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Funcion para el comienzo de la aceleracion del motor paso a paso          
 *
 * Note:            None
 *****************************************************************************/
void Denboragailu_0_on (struct CONTROLMOVIMIENTO *pointermovimiento)
{

mLED_4_On();

if (controlMovimiento.tipoMovimiento==MEDIOPASO)
{
OpenTimer0( TIMER_INT_ON &     	   //Interrupt enabled
                T0_16BIT &         //Set timer0 as 16 bit mode.
                T0_SOURCE_INT &    //choose Internal clock source (TOSC)
                T0_PS_1_64);         //Prescale Value: 1:64  
}
else
{
OpenTimer0( TIMER_INT_ON &     	   //Interrupt enabled
                T0_16BIT &         //Set timer0 as 16 bit mode.
                T0_SOURCE_INT &    //choose Internal clock source (TOSC)
                T0_PS_1_128         //Prescale Value: 1:128
               );  
}

contadorAceleracion=0; //Inicializamos el contador de pasos de aceleracion/deceleracion
aceleracion=1;		//Indicamos que el aceleracion(0 indica deceleracion)
WriteTimer0(60000); //Un simple retraso de 500ms hasta que entre en el bucle
  		
}

//end Denboragailu_0_on



/******************************************************************************
 * Function:        void Denboragailu_0_off (void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Funcion para el comienzo de la deceleracion del motor
 *
 * Note:            None
 *****************************************************************************/
void Denboragailu_0_off ()
{
contadorDeceleracion=controlMovimiento.pasosDeceleracion;
aceleracion=0; //Indicamos que ahora debe decelerar

}


/******************************************************************************
 * Function:        void ProcessIO(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for other user routines.
 *                  It is a mixture of both USB and non-USB tasks.
 *
 * Note:            None
 *****************************************************************************/
void ProcessIO(void)
{   
BlinkUSBStatus();
    // User Application USB tasks
    if((usb_device_state < CONFIGURED_STATE)||(UCONbits.SUSPND==1)) return;
    
    ServiceRequests();

}//end ProcessIO


void ServiceRequests(void)
{
   
byte index;
	
    
    if(USBGenRead((byte*)&dataPacket,sizeof(dataPacket)))
    {
        counter = 0;

        switch(dataPacket.CMD)
        {
            case READ_VERSION:
                //dataPacket._byte[1] is len
                dataPacket._byte[2] = MINOR_VERSION;
                dataPacket._byte[3] = MAJOR_VERSION;
                counter=0x04;
                break;
			 case ID_BOARD:
                counter = 0x01;
                if(dataPacket.ID == 0)
                {
                    mLED_3_Off();mLED_4_Off(); 
                }
                else if(dataPacket.ID == 1)
                {
                    mLED_3_Off();mLED_4_On();
                }
                else if(dataPacket.ID == 2)
                {
                    mLED_3_On();mLED_4_Off();
                }
                else if(dataPacket.ID == 3)
                {
                    mLED_3_On();mLED_4_On();
                }
                else
                    counter = 0x00;
                break;

			case DENBORAGAILUA_0_ON:
			
				ContadorInicialTemp0(dataPacket._byte+1);
				controlMovimiento.sentidoGiro=dataPacket._byte[5];
				controlMovimiento.pasosAceleracion=dataPacket._byte[6];//slo 1 byte=256 pasos maximo
				controlMovimiento.pasosDeceleracion=dataPacket._byte[7];//slo 1 byte=256 pasos maximo
				controlMovimiento.tipoMovimiento=dataPacket._byte[8];
				PWMdutyCorrienteFase(dataPacket._byte+9); //funcion para introducir los valores de las corrientes
				pointerControlmovimiento=&controlMovimiento; //Damos a pointerControlmovimiento la direccion de la variable controlMovimiento
				Denboragailu_0_on(pointerControlmovimiento); 
 				counter=0x01;
			
            break;
			case DENBORAGAILUA_0_OFF:		
 				Denboragailu_0_off();
				counter=0x01;
				break;

            case RESET:
                Reset();
                break;
            case VELOCIDAD:
				Leer_veloc();				
   			    dataPacket._byte[1] = velocL;	
                dataPacket._byte[2] = velocH;
                counter=0x03;            
            default:
                break;
                
        }//end switch()   
        if(counter != 0)
        {
            if(!mUSBGenTxIsBusy())
                USBGenWrite((byte*)&dataPacket,counter);
        }//end if
    }//end if

}//end ServiceRequests

/******************************************************************************
 * Function:        void BlinkUSBStatus(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        BlinkUSBStatus turns on and off LEDs corresponding to
 *                  the USB device state.
 *
 * Note:            mLED macros can be found in io_cfg.h
 *                  usb_device_state is declared in usbmmap.c and is modified
 *                  in usbdrv.c, usbctrltrf.c, and usb9.c
 *****************************************************************************/
void BlinkUSBStatus(void)
{
    static word led_count=0;
    
    if(led_count == 0)led_count = 10000U;
    led_count--;

    #define mLED_Both_Off()         {mLED_1_Off();mLED_2_Off();}
    #define mLED_Both_On()          {mLED_1_On();mLED_2_On();}
    #define mLED_Only_1_On()        {mLED_1_On();mLED_2_Off();}
    #define mLED_Only_2_On()        {mLED_1_Off();mLED_2_On();}

    if(UCONbits.SUSPND == 1)
    {
        if(led_count==0)
        {
            mLED_1_Toggle();
            mLED_2 = mLED_1;        // Both blink at the same time
        }//end if
    }
    else
    {
        if(usb_device_state == DETACHED_STATE)
        {
            mLED_Both_Off();
            
        }
        else if(usb_device_state == ATTACHED_STATE)
        {
            mLED_Both_On();
        }
        else if(usb_device_state == POWERED_STATE)
        {
            mLED_Only_1_On();
        }
        else if(usb_device_state == DEFAULT_STATE)
        {
            mLED_Only_2_On();
        }
        else if(usb_device_state == ADDRESS_STATE)
        {
            if(led_count == 0)
            {
                mLED_1_Toggle();
                mLED_2_Off();
            }//end if
        }
        else if(usb_device_state == CONFIGURED_STATE)
        {
            if(led_count==0)
            {
                mLED_1_Toggle();
                mLED_2 = !mLED_1;       // Alternate blink                
            }//end if
        }//end if(...)
    }//end if(UCONbits.SUSPND...)

}//end BlinkUSBStatus

/** EOF user.c ***************************************************************/
