//Aadimos los namespace que necesitaremos
using System;
using System.Runtime.InteropServices;
using WindowsApplication3;
using PVOID = System.IntPtr;
using DWORD = System.UInt32;


//Creamos nuiiestro namespace usb_api
namespace usb_api
{
    //Creamos la clase usb_interface, la definimos como unsafe para poder utilizar
    //punteros en ella
    unsafe public class usb_interface
    {
        #region  String Definitions of Pipes and VID_PID
        //string vid_pid_boot= "vid_04d8&pid_000b";    // Bootloader vid_pid ID
        string vid_pid_norm = "vid_04d8&pid_000c";

        string out_pipe = "\\MCHP_EP1"; // Define End Points
        string in_pipe = "\\MCHP_EP1";
        #endregion

        #region Imported DLL functions from mpusbapi.dll
        [DllImport("mpusbapi.dll")]
        private static extern DWORD _MPUSBGetDLLVersion();
        [DllImport("mpusbapi.dll")]
        private static extern DWORD _MPUSBGetDeviceCount(string pVID_PID);
        [DllImport("mpusbapi.dll")]
        private static extern void* _MPUSBOpen(DWORD instance, string pVID_PID, string pEP, DWORD dwDir, DWORD dwReserved);
        [DllImport("mpusbapi.dll")]
        private static extern DWORD _MPUSBRead(void* handle, void* pData, DWORD dwLen, DWORD* pLength, DWORD dwMilliseconds);
        [DllImport("mpusbapi.dll")]
        private static extern DWORD _MPUSBWrite(void* handle, void* pData, DWORD dwLen, DWORD* pLength, DWORD dwMilliseconds);
        [DllImport("mpusbapi.dll")]
        private static extern DWORD _MPUSBReadInt(void* handle, DWORD dwLen, DWORD* pLength, DWORD dwMilliseconds);
        [DllImport("mpusbapi.dll")]
        private static extern bool _MPUSBClose(void* handle);
        #endregion

        void* myOutPipe;
        void* myInPipe;

        private void OpenPipes()
        {
            DWORD selection = 0; // Selects the device to connect to, in this example it is assumed you will only have one device per vid_pid connected.

            myOutPipe = _MPUSBOpen(selection, vid_pid_norm, out_pipe, 0, 0);
            myInPipe = _MPUSBOpen(selection, vid_pid_norm, in_pipe, 1, 0);
        }

        private void ClosePipes()
        {
            _MPUSBClose(myOutPipe);
            _MPUSBClose(myInPipe);
        }
        //SendReceivePacket(send_buf,4,receive_buf,&RecvLength) == 1
        private DWORD SendReceivePacket(byte* SendData, DWORD SendLength, byte* ReceiveData, DWORD* ReceiveLength)
        {
            uint SendDelay = 1000;
            uint ReceiveDelay = 1000;

            DWORD SentDataLength;
            DWORD ExpectedReceiveLength = *ReceiveLength;

            OpenPipes();

            if (_MPUSBWrite(myOutPipe, (void*)SendData, SendLength, &SentDataLength, SendDelay) == 1)
            {

                if (_MPUSBRead(myInPipe, (void*)ReceiveData, ExpectedReceiveLength, ReceiveLength, ReceiveDelay) == 1)
                {
                    if (*ReceiveLength == ExpectedReceiveLength)
                    {
                        ClosePipes();
                        return 1;   // Success!
                    }
                    else if (*ReceiveLength < ExpectedReceiveLength)
                    {
                        ClosePipes();
                        return 2;   // Partially failed, incorrect receive length
                    }
                }
            }
            ClosePipes();
            return 0;  // Operation Failed
        }

        public DWORD GetDLLVersion()
        {
            return _MPUSBGetDLLVersion();
        }

        public DWORD GetDeviceCount(string Vid_Pid)
        {
            return _MPUSBGetDeviceCount(Vid_Pid);
        }
           
        public short datos_velocidad()
        {
            byte* send_buf = stackalloc byte[64];
            byte* receive_buf = stackalloc byte[64];

            short ValorVelocidad;
            byte[] byteArray2 = { 0, 0 };
            DWORD RecvLength = (byte)3;
            send_buf[0] = 0xA3;			//Orden al micro de que coja la velocidad 
            //send_buf[1] = (byte)num1; 
            //send_buf[2] = (byte)num2;

            if (SendReceivePacket(send_buf, 1, receive_buf, &RecvLength) == 1)
            {

                byteArray2[0] = *(receive_buf + 1);
                byteArray2[1] = *(receive_buf + 2);
                ValorVelocidad = BitConverter.ToInt16(byteArray2, 0);
                //ValorVelocidad= 100;
                return ValorVelocidad; //Valor devuelto por el micro en formato int16
            }

            else  return 2500;

      
        }  

        public uint controlMovUsb(ControlMovimiento e) 
        {
            uint Temp=0;
            
            byte* send_buf = stackalloc byte[64];
            byte* receive_buf = stackalloc byte[64];
          
            byte[] contadorAcelTemp0 = new byte[2];
            byte[] contadorDecelTemp0 = new byte[2];
            byte[] sendcurrentA = new byte[2];
            byte[] sendcurrentB = new byte[2];
            contadorAcelTemp0 = BitConverter.GetBytes(e.contadorInicialAcelTemp0);
            contadorDecelTemp0 = BitConverter.GetBytes(e.contadorInicialDecelTemp0);
            sendcurrentA = BitConverter.GetBytes(e.valueCorrienteFaseA);
            sendcurrentB = BitConverter.GetBytes(e.valueCorrienteFaseB);

            DWORD RecvLength = (byte)3;
            send_buf[0] =0xA0;			
            send_buf[1] = contadorAcelTemp0[0];
            send_buf[2] = contadorAcelTemp0[1];
            send_buf[3] = contadorDecelTemp0[0];
            send_buf[4] = contadorDecelTemp0[1];
            send_buf[5] = e.byteSentidoGiro;
            send_buf[6] = e.valuePasosAceleracion;
            send_buf[7] = e.valuePasosDeceleracion;
            send_buf[8] = e.byteTipoMovimiento;
            send_buf[9]  = sendcurrentA[0];
            send_buf[10] = sendcurrentA[1];
            send_buf[11] = sendcurrentB[0];
            send_buf[12] = sendcurrentB[1];

            if (SendReceivePacket(send_buf, 13, receive_buf, &RecvLength) == 1)
            {
                Temp = (uint)receive_buf[1];
            }
            return Temp;
        }

        public uint Stop() {

            byte* send_buf = stackalloc byte[64];
            byte* receive_buf = stackalloc byte[64];

            DWORD RecvLength = 3;

            send_buf[0] = 0xA1;			//Command for TIMER0_OFF  
            

            if (SendReceivePacket(send_buf, 1, receive_buf, &RecvLength) == 1)
            {
                if (RecvLength == 1 && receive_buf[0] == 0xA1)
                {
                    return 0;
                }
                else
                {
                    return 2;
                }
            }
            else
            {
                    return 1;
            }
        }

        }
    }



